@@ -112,6 +112,11 @@ namespace Opm {
         void reset_actnum(const std::vector<int>& new_actnum);
         void pruneDeactivatedAquiferConnections(const std::vector<std::size_t>& deactivated_cells);
         void loadRestartAquifers(const RestartIO::RstAquifer& aquifers);
+        // TODO: it is possible that the aquifer are opened through SCHEDULE and not specified in the SOLUTION section
+        // For the ease of the implementation, we create inactive aquifer in the AquiferConfig.
+        // At the moment, it only works for Constant Flux Aquifers(AQUFLUX) as indicated by the function name
+        // When we know and decide to handle the same for AQUFETP and AQUCT, this part will be refactored
+        void appendAqufluxSchedule(const std::unordered_set<int>& ids);
 
         template<class Serializer>
         void serializeOp(Serializer& serializer)
