@@ -53,7 +53,7 @@ namespace Opm {
 
     class EclipseGrid : public GridDims {
     public:
-        EclipseGrid() = default;
+        EclipseGrid();
         explicit EclipseGrid(const std::string& filename);
 
         /*
@@ -108,14 +108,15 @@ namespace Opm {
           the theta keywords entered sum up to exactly 360 degrees!
         */
 
-        bool circle( ) const;
-        bool isPinchActive( ) const;
-        double getPinchThresholdThickness( ) const;
-        PinchMode::ModeEnum getPinchOption( ) const;
-        PinchMode::ModeEnum getMultzOption( ) const;
-        PinchMode::ModeEnum getPinchGapMode( ) const;
+        bool circle() const;
+        bool isPinchActive() const;
+        double getPinchThresholdThickness() const;
+        PinchMode getPinchOption() const;
+        PinchMode getMultzOption() const;
+        PinchMode getPinchGapMode() const;
+        double getPinchMaxEmptyGap() const;
 
-        MinpvMode::ModeEnum getMinpvMode() const;
+        MinpvMode getMinpvMode() const;
         const std::vector<double>& getMinpvVector( ) const;
 
         /*
@@ -209,24 +210,25 @@ namespace Opm {
 
         bool equal(const EclipseGrid& other) const;
         static bool hasDVDEPTHZKeywords(const Deck&);
-        
+
         /*
-          For ALugrid we can *only* use the keyword <DXV, DXYV, DZV, DEPTHZ> so to  
+          For ALugrid we can *only* use the keyword <DXV, DXYV, DZV, DEPTHZ> so to
           initialize a Regular Cartesian Grid; further we need equidistant mesh
-          spacing in each direction to initialize ALuGrid (mandatory for  
+          spacing in each direction to initialize ALuGrid (mandatory for
           mesh refinement!).
         */
-        
+
         static bool hasEqualDVDEPTHZ(const Deck&);
         static bool allEqual(const std::vector<double> &v);
 
     private:
         std::vector<double> m_minpvVector;
-        MinpvMode::ModeEnum m_minpvMode;
+        MinpvMode m_minpvMode;
         std::optional<double> m_pinch;
-        PinchMode::ModeEnum m_pinchoutMode;
-        PinchMode::ModeEnum m_multzMode;
-        PinchMode::ModeEnum m_pinchGapMode;
+        PinchMode m_pinchoutMode;
+        PinchMode m_multzMode;
+        PinchMode m_pinchGapMode;
+        double    m_pinchMaxEmptyGap;
 
         mutable std::optional<std::vector<double>> active_volume;
 
@@ -236,13 +238,18 @@ namespace Opm {
         bool m_useActnumFromGdfile = false;
 
         // Input grid data.
+        mutable std::optional<std::vector<double>> m_input_zcorn;
+        mutable std::optional<std::vector<double>> m_input_coord;
+
         std::vector<double> m_zcorn;
         std::vector<double> m_coord;
+
+
         std::vector<int> m_actnum;
         std::optional<MapAxes> m_mapaxes;
 
         // Mapping to/from active cells.
-        int m_nactive;
+        int m_nactive {};
         std::vector<int> m_active_to_global;
         std::vector<int> m_global_to_active;
         // Numerical aquifer cells, needs to be active
