@@ -131,6 +131,8 @@ static const std::unordered_map<std::string, keyword_info<double>> double_keywor
                                                                                       {"TEMPI",   keyword_info<double>{}.unit_string("Temperature")},
                                                                                       {"THCONR",  keyword_info<double>{}.unit_string("Energy/AbsoluteTemperature*Length*Time")},
                                                                                       {"THCONSF", keyword_info<double>{}},
+                                                                                      {"HEATCR",  keyword_info<double>{}.unit_string("Energy/ReservoirVolume*AbsoluteTemperature")},
+                                                                                      {"HEATCRT", keyword_info<double>{}.unit_string("Energy/ReservoirVolume*AbsoluteTemperature*AbsoluteTemperature")},
                                                                                       {"THCROCK", keyword_info<double>{}.unit_string("Energy/AbsoluteTemperature*Length*Time")},
                                                                                       {"THCOIL",  keyword_info<double>{}.unit_string("Energy/AbsoluteTemperature*Length*Time")},
                                                                                       {"THCGAS",  keyword_info<double>{}.unit_string("Energy/AbsoluteTemperature*Length*Time")},
@@ -140,7 +142,7 @@ static const std::unordered_map<std::string, keyword_info<double>> double_keywor
                                                                                       {"MULTY",   keyword_info<double>{}.init(1.0).mult(true)},
                                                                                       {"MULTY-",  keyword_info<double>{}.init(1.0).mult(true)},
                                                                                       {"MULTZ",   keyword_info<double>{}.init(1.0).mult(true).global_kw(true)},
-                                                                                      {"MULTZ-",  keyword_info<double>{}.init(1.0).mult(true)}};
+                                                                                      {"MULTZ-",  keyword_info<double>{}.init(1.0).mult(true).global_kw(true)}};
 
 static const std::unordered_map<std::string, keyword_info<int>> int_keywords = {{"ACTNUM",  keyword_info<int>{}.init(1)},
                                                                                 {"FLUXNUM", keyword_info<int>{}},
@@ -167,7 +169,7 @@ static const std::unordered_map<std::string, keyword_info<double>> double_keywor
                                                                                       {"MULTY",   keyword_info<double>{}.init(1.0).mult(true)},
                                                                                       {"MULTY-",  keyword_info<double>{}.init(1.0).mult(true)},
                                                                                       {"MULTZ",   keyword_info<double>{}.init(1.0).mult(true).global_kw(true)},
-                                                                                      {"MULTZ-",  keyword_info<double>{}.init(1.0).mult(true)}};
+                                                                                      {"MULTZ-",  keyword_info<double>{}.init(1.0).mult(true).global_kw(true)}};
 
 static const std::unordered_map<std::string, keyword_info<int>> int_keywords = {};
 }
@@ -247,6 +249,9 @@ static const std::unordered_map<std::string, keyword_info<int>> int_keywords = {
                                                                                 {"KRNUMX",   keyword_info<int>{}},
                                                                                 {"KRNUMY",   keyword_info<int>{}},
                                                                                 {"KRNUMZ",   keyword_info<int>{}},
+                                                                                {"IMBNUMX",   keyword_info<int>{}},
+                                                                                {"IMBNUMY",   keyword_info<int>{}},
+                                                                                {"IMBNUMZ",   keyword_info<int>{}},
                                                                                 };
 }
 
@@ -280,7 +285,7 @@ static const std::unordered_map<std::string, keyword_info<double>> double_keywor
                                                                                       {"MULTY",   keyword_info<double>{}.init(1.0).mult(true)},
                                                                                       {"MULTY-",  keyword_info<double>{}.init(1.0).mult(true)},
                                                                                       {"MULTZ",   keyword_info<double>{}.init(1.0).mult(true).global_kw(true)},
-                                                                                      {"MULTZ-",  keyword_info<double>{}.init(1.0).mult(true)}};
+                                                                                      {"MULTZ-",  keyword_info<double>{}.init(1.0).mult(true).global_kw(true)}};
 
 static const std::unordered_map<std::string, keyword_info<int>> int_keywords = {{"ROCKNUM",   keyword_info<int>{}}};
 
@@ -509,7 +514,7 @@ public:
     bool operator==(const FieldProps& other) const;
     static bool rst_cmp(const FieldProps& full_arg, const FieldProps& rst_arg);
 
-    const Fieldprops::TranMap& getTran() const
+    const std::unordered_map<std::string,Fieldprops::TranCalculator>& getTran() const
     {
         return tran;
     }
@@ -574,7 +579,7 @@ private:
     std::unordered_map<std::string, Fieldprops::FieldData<int>> int_data;
     std::unordered_map<std::string, Fieldprops::FieldData<double>> double_data;
 
-    Fieldprops::TranMap tran;
+    std::unordered_map<std::string,Fieldprops::TranCalculator> tran;
 };
 
 }
