@@ -20,16 +20,19 @@
 #define FIELDPROPS_MANAGER_HPP
 
 #include <memory>
-#include <vector>
+#include <string>
 #include <unordered_map>
-#include <opm/input/eclipse/EclipseState/Grid/TranCalculator.hpp>
-#include <opm/input/eclipse/EclipseState/Grid/FieldData.hpp>
+#include <vector>
 
 namespace Opm {
 
 class EclipseGrid;
 class Deck;
 class DeckKeyword;
+namespace Fieldprops {
+class TranCalculator;
+template<typename T> struct FieldData;
+}
 class FieldProps;
 class Phases;
 class TableManager;
@@ -230,7 +233,7 @@ public:
 
     void apply_numerical_aquifers(const NumericalAquifers& aquifers);
 
-    const Fieldprops::TranMap& getTran() const;
+    const std::unordered_map<std::string,Fieldprops::TranCalculator>& getTran() const;
 
 private:
     /*
