@@ -22,6 +22,7 @@
 
 #include <vector>
 #include <cstddef>
+#include <optional>
 
 #include <opm/input/eclipse/EclipseState/Grid/FaceDir.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/GridDims.hpp>
@@ -34,7 +35,9 @@ class DeckRecord;
 
 enum class BCType {
      RATE,
-     FREE
+     FREE,
+     DIRICHLET,
+     THERMAL
 };
 
 enum class BCComponent {
@@ -58,6 +61,8 @@ public:
         FaceDir::DirEnum dir;
         BCComponent component;
         double rate;
+        std::optional<double> pressure;
+        std::optional<double> temperature;
 
         BCFace() = default;
         explicit BCFace(const DeckRecord& record, const GridDims& grid);
@@ -79,6 +84,8 @@ public:
             serializer(dir);
             serializer(component);
             serializer(rate);
+            serializer(pressure);
+            serializer(temperature);
         }
     };
 
