@@ -132,6 +132,9 @@ namespace Opm {
         const TableContainer& getMsfnTables() const;
         const TableContainer& getTlpmixpaTables() const;
 
+        const TableContainer& getWsfTables() const;
+        const TableContainer& getGsfTables() const;
+
         const JFunc& getJFunc() const;
 
         const std::vector<PvtgTable>& getPvtgTables() const;
@@ -161,6 +164,8 @@ namespace Opm {
         const PvcdoTable& getPvcdoTable() const;
         const DensityTable& getDensityTable() const;
         const DiffCoeffTable& getDiffusionCoefficientTable() const;
+        const DiffCoeffWatTable& getDiffusionCoefficientWaterTable() const;
+        const DiffCoeffGasTable& getDiffusionCoefficientGasTable() const;
         const PlyvmhTable& getPlyvmhTable() const;
         const RockTable& getRockTable() const;
         const ViscrefTable& getViscrefTable() const;
@@ -217,6 +222,8 @@ namespace Opm {
             serializer(m_pvcdoTable);
             serializer(m_densityTable);
             serializer(m_diffCoeffTable);
+            serializer(m_diffCoeffWatTable);
+            serializer(m_diffCoeffGasTable);
             serializer(m_plyvmhTable);
             serializer(m_rockTable);
             serializer(m_plmixparTable);
@@ -361,6 +368,8 @@ namespace Opm {
         PvcdoTable m_pvcdoTable;
         DensityTable m_densityTable;
         DiffCoeffTable m_diffCoeffTable;
+        DiffCoeffWatTable m_diffCoeffWatTable;
+        DiffCoeffGasTable m_diffCoeffGasTable;
         PlyvmhTable m_plyvmhTable;
         RockTable m_rockTable;
         PlmixparTable m_plmixparTable;
