@@ -25,46 +25,56 @@
 #include <memory>
 #include <optional>
 #include <iosfwd>
+#include <set>
 #include <string>
 #include <unordered_map>
 #include <utility>
 #include <vector>
 
 #include <opm/input/eclipse/EclipseState/Runspec.hpp>
-#include <opm/input/eclipse/Parser/InputErrorAction.hpp>
-#include <opm/input/eclipse/Python/Python.hpp>
-#include <opm/input/eclipse/Schedule/GasLiftOpt.hpp>
 #include <opm/input/eclipse/Schedule/Group/Group.hpp>
-#include <opm/input/eclipse/Schedule/Group/GuideRateConfig.hpp>
 #include <opm/input/eclipse/Schedule/MessageLimits.hpp>
-#include <opm/input/eclipse/Schedule/Network/ExtNetwork.hpp>
-#include <opm/input/eclipse/Schedule/RPTConfig.hpp>
 #include <opm/input/eclipse/Schedule/ScheduleDeck.hpp>
 #include <opm/input/eclipse/Schedule/ScheduleState.hpp>
 #include <opm/input/eclipse/Schedule/Well/PAvg.hpp>
-#include <opm/input/eclipse/Schedule/Well/Well.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellTestConfig.hpp>
 #include <opm/input/eclipse/Schedule/WriteRestartFileEvents.hpp>
 #include <opm/input/eclipse/Schedule/CompletedCells.hpp>
-#include <opm/input/eclipse/Schedule/Action/SimulatorUpdate.hpp>
 #include <opm/input/eclipse/Schedule/Action/WGNames.hpp>
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
 
 namespace Opm
 {
+    namespace Action {
+        class ActionX;
+        class PyAction;
+        class State;
+    }
     class ActiveGridCells;
     class Deck;
     class DeckKeyword;
     class DeckRecord;
     class EclipseState;
-    class FieldPropsManager;
-    class GTNode;
-    class ParseContext;
-    class SCHEDULESection;
-    class SummaryState;
     class ErrorGuard;
+    class FieldPropsManager;
+    class GasLiftOpt;
+    class GTNode;
+    class GuideRateConfig;
+    class GuideRateModel;
+    enum class InputErrorAction;
+    class ParseContext;
+    class Python;
+    class RPTConfig;
+    class SCHEDULESection;
+    struct SimulatorUpdate;
+    class SummaryState;
+    class TracerConfig;
     class UDQConfig;
+    class Well;
+    enum class WellGasInflowEquation;
     class WellMatcher;
+    enum class WellProducerCMode;
+    enum class WellStatus;
+    class WellTestConfig;
 
     namespace RestartIO { struct RstState; }
 
@@ -111,32 +121,36 @@ namespace Opm
         }
 
 
-        static ScheduleStatic serializationTestObject() {
-            auto python = std::make_shared<Python>(Python::Enable::OFF);
-            ScheduleStatic st(python);
-            st.m_deck_message_limits = MessageLimits::serializationTestObject();
-            st.m_runspec = Runspec::serializationTestObject();
-            st.m_unit_system = UnitSystem::newFIELD();
-            st.m_input_path = "Some/funny/path";
-            st.rst_config = RSTConfig::serializationTestObject();
-            st.rst_info = ScheduleRestartInfo::serializationTestObject();
-            return st;
-        }
+        static ScheduleStatic serializationTestObject();
 
-        bool operator==(const ScheduleStatic& other) const {
-            return this->m_input_path == other.m_input_path &&
-                   this->m_deck_message_limits == other.m_deck_message_limits &&
-                   this->m_unit_system == other.m_unit_system &&
-                   this->rst_config == other.rst_config &&
-                   this->rst_info == other.rst_info &&
-                   this->gaslift_opt_active == other.gaslift_opt_active &&
-                   this->m_runspec == other.m_runspec;
-        }
+        bool operator==(const ScheduleStatic& other) const;
     };
 
 
     class Schedule {
     public:
+
+        struct PairComp
+        {
+            bool operator()(const std::pair<std::string,KeywordLocation>& pair,
+                            const std::string& str) const
+            {
+                return std::get<0>(pair) < str;
+            }
+            bool operator()(const std::pair<std::string,KeywordLocation>& pair1,
+                            const std::pair<std::string,KeywordLocation>& pair2) const
+            {
+                return std::get<0>(pair1) < std::get<0>(pair2);
+            }
+            bool operator()(const std::string& str,
+                            const std::pair<std::string,KeywordLocation>& pair) const
+            {
+                return str < std::get<0>(pair);
+            }
+        };
+
+        using WelSegsSet = std::set<std::pair<std::string,KeywordLocation>,PairComp>;
+
         Schedule() = default;
         explicit Schedule(std::shared_ptr<const Python> python_handle);
         Schedule(const Deck& deck,
@@ -247,6 +261,8 @@ namespace Opm
         const Well& getWell(std::size_t well_index, std::size_t timeStep) const;
         const Well& getWell(const std::string& wellName, std::size_t timeStep) const;
         const Well& getWellatEnd(const std::string& well_name) const;
+        // get the list of the constant flux aquifer specified in the whole schedule
+        std::unordered_set<int> getAquiferFluxSchedule() const;
         std::vector<Well> getWells(std::size_t timeStep) const;
         std::vector<Well> getWellsatEnd() const;
         void shut_well(const std::string& well_name, std::size_t report_step);
@@ -256,7 +272,7 @@ namespace Opm
 
         std::vector<const Group*> getChildGroups2(const std::string& group_name, std::size_t timeStep) const;
         std::vector<Well> getChildWells2(const std::string& group_name, std::size_t timeStep) const;
-        Well::ProducerCMode getGlobalWhistctlMmode(std::size_t timestep) const;
+        WellProducerCMode getGlobalWhistctlMmode(std::size_t timestep) const;
 
         const UDQConfig& getUDQConfig(std::size_t timeStep) const;
         void evalAction(const SummaryState& summary_state, std::size_t timeStep);
@@ -489,6 +505,7 @@ namespace Opm
 
     private:
         struct HandlerContext {
+
             const ScheduleBlock& block;
             const DeckKeyword& keyword;
             const std::size_t currentStep;
@@ -499,8 +516,12 @@ namespace Opm
             SimulatorUpdate * sim_update;
             const std::unordered_map<std::string, double> * target_wellpi;
             std::unordered_map<std::string, double>* wpimult_global_factor;
+            WelSegsSet *welsegs_wells;
+            std::set<std::string>*compsegs_wells;
             const ScheduleGrid& grid;
 
+            /// \param welsegs_wells All wells with a WELSEGS entry for checks.
+            /// \param compegs_wells All wells with a COMPSEGS entry for checks.
             HandlerContext(const ScheduleBlock& block_,
                            const DeckKeyword& keyword_,
                            const ScheduleGrid& grid_,
@@ -511,7 +532,9 @@ namespace Opm
                            ErrorGuard& errors_,
                            SimulatorUpdate * sim_update_,
                            const std::unordered_map<std::string, double> * target_wellpi_,
-                           std::unordered_map<std::string, double>* wpimult_global_factor_)
+                           std::unordered_map<std::string, double>* wpimult_global_factor_,
+                           WelSegsSet* welsegs_wells_,
+                           std::set<std::string>* compsegs_wells_)
             : block(block_)
             , keyword(keyword_)
             , currentStep(currentStep_)
@@ -522,12 +545,25 @@ namespace Opm
             , sim_update(sim_update_)
             , target_wellpi(target_wellpi_)
             , wpimult_global_factor(wpimult_global_factor_)
+            , welsegs_wells(welsegs_wells_)
+            , compsegs_wells(compsegs_wells_)
             , grid(grid_)
             {}
 
-            void affected_well(const std::string& well_name) {
-                if (this->sim_update)
-                    this->sim_update->affected_wells.insert(well_name);
+            void affected_well(const std::string& well_name);
+
+            /// \brief Mark that the well occured in a  WELSEGS keyword
+            void welsegs_handled(const std::string& well_name)
+            {
+                if (welsegs_wells)
+                    welsegs_wells->insert({well_name, keyword.location()});
+            }
+
+            /// \brief Mark that the well occured in a  COMPSEGS keyword
+            void compsegs_handled(const std::string& well_name)
+            {
+                if (compsegs_wells)
+                    compsegs_wells->insert(well_name);
             }
 
         };
@@ -560,7 +596,7 @@ namespace Opm
                      bool allowCrossFlow,
                      bool automaticShutIn,
                      int pvt_table,
-                     Well::GasInflowEquation gas_inflow,
+                     WellGasInflowEquation gas_inflow,
                      std::size_t timeStep,
                      Connection::Order wellConnectionOrder);
         bool updateWPAVE(const std::string& wname, std::size_t report_step, const PAvg& pavg);
@@ -568,7 +604,7 @@ namespace Opm
         void updateGuideRateModel(const GuideRateModel& new_model, std::size_t report_step);
         GTNode groupTree(const std::string& root_node, std::size_t report_step, std::size_t level, const std::optional<std::string>& parent_name) const;
         bool checkGroups(const ParseContext& parseContext, ErrorGuard& errors);
-        bool updateWellStatus( const std::string& well, std::size_t reportStep, Well::Status status, std::optional<KeywordLocation> = {});
+        bool updateWellStatus( const std::string& well, std::size_t reportStep, WellStatus status, std::optional<KeywordLocation> = {});
         void addWellToGroup( const std::string& group_name, const std::string& well_name , std::size_t timeStep);
         void iterateScheduleSection(std::size_t load_start,
                                     std::size_t load_end,
@@ -586,6 +622,8 @@ namespace Opm
         void addWell(const std::string& wellName, const DeckRecord& record, std::size_t timeStep, Connection::Order connection_order);
         void checkIfAllConnectionsIsShut(std::size_t currentStep);
         void end_report(std::size_t report_step);
+        /// \param welsegs_wells All wells with a WELSEGS entry for checks.
+        /// \param compegs_wells All wells with a COMPSEGS entry for checks.
         void handleKeyword(std::size_t currentStep,
                            const ScheduleBlock& block,
                            const DeckKeyword& keyword,
@@ -596,12 +634,14 @@ namespace Opm
                            bool actionx_mode,
                            SimulatorUpdate* sim_update,
                            const std::unordered_map<std::string, double>* target_wellpi,
-                           std::unordered_map<std::string, double>* wpimult_global_factor = nullptr);
+                           std::unordered_map<std::string, double>* wpimult_global_factor = nullptr,
+                           WelSegsSet* welsegs_wells = nullptr,
+                           std::set<std::string>* compsegs_wells = nullptr);
 
         void prefetch_cell_properties(const ScheduleGrid& grid, const DeckKeyword& keyword);
         void store_wgnames(const DeckKeyword& keyword);
         std::vector<std::string> wellNames(const std::string& pattern, const HandlerContext& context);
-        std::vector<std::string> wellNames(const std::string& pattern, std::size_t timeStep, const std::vector<std::string>& matching_wells, InputError::Action error_action, ErrorGuard& errors, const KeywordLocation& location) const;
+        std::vector<std::string> wellNames(const std::string& pattern, std::size_t timeStep, const std::vector<std::string>& matching_wells, InputErrorAction error_action, ErrorGuard& errors, const KeywordLocation& location) const;
         void invalidNamePattern( const std::string& namePattern, const HandlerContext& context) const;
         static std::string formatDate(std::time_t t);
         std::string simulationDays(std::size_t currentStep) const;
@@ -635,11 +675,15 @@ namespace Opm
 
         // Normal keyword handlers -- in KeywordHandlers.cpp
 
+        void handleAQUCT     (HandlerContext&);
+        void handleAQUFETP   (HandlerContext&);
+        void handleAQUFLUX   (HandlerContext&);
         void handleBRANPROP  (HandlerContext&);
         void handleCOMPDAT   (HandlerContext&);
         void handleCOMPLUMP  (HandlerContext&);
         void handleCOMPORD   (HandlerContext&);
         void handleCOMPSEGS  (HandlerContext&);
+        void handleCOMPTRAJ  (HandlerContext&);
         void handleDRSDT     (HandlerContext&);
         void handleDRSDTCON  (HandlerContext&);
         void handleDRSDTR    (HandlerContext&);
@@ -687,6 +731,7 @@ namespace Opm
         void handleWELSEGS   (HandlerContext&);
         void handleWELSPECS  (HandlerContext&);
         void handleWELTARG   (HandlerContext&);
+        void handleWELTRAJ   (HandlerContext&);
         void handleWFOAM     (HandlerContext&);
         void handleWGRUPCON  (HandlerContext&);
         void handleWHISTCTL  (HandlerContext&);
