@@ -139,6 +139,7 @@ namespace Opm {
         constexpr const double gravity = 9.80665 * meter/square(second);
         /// @}
 
+        constexpr const double mol = 1;
         // --------------------------------------------------------------
         // Derived units and conversions
         // --------------------------------------------------------------
@@ -279,6 +280,7 @@ namespace Opm {
         constexpr const double Timestep             = day;
         constexpr const double SurfaceTension       = dyne/(centi*meter);
         constexpr const double Energy               = kilo*joule;
+        constexpr const double Moles                = kilo*mol;
     }
 
 
@@ -310,6 +312,8 @@ namespace Opm {
         constexpr const double Timestep             = day;
         constexpr const double SurfaceTension       = dyne/(centi*meter);
         constexpr const double Energy               = btu;
+        constexpr const double Moles                = kilo*pound*mol;
+
     }
 
 
@@ -341,6 +345,8 @@ namespace Opm {
         constexpr const double Timestep             = hour;
         constexpr const double SurfaceTension       = dyne/(centi*meter);
         constexpr const double Energy               = joule;
+        constexpr const double Moles                = mol;
+
     }
 
 
@@ -372,6 +378,7 @@ namespace Opm {
         constexpr const double Timestep             = day;
         constexpr const double SurfaceTension       = dyne/(centi*meter);
         constexpr const double Energy               = kilo*joule;
+        constexpr const double Moles                = kilo*mol;
     }
 }
 
