@@ -59,7 +59,6 @@
 #include <cstring>
 #include <functional>
 #include <initializer_list>
-#include <iostream>
 #include <numeric>
 #include <tuple>
 
@@ -91,16 +90,25 @@ void apply_GRIDUNIT(const UnitSystem& deck_units, const UnitSystem& grid_units,
 }
 
 }
+EclipseGrid::EclipseGrid()
+    : GridDims(),
+      m_minpvMode(MinpvMode::Inactive),
+      m_pinchoutMode(PinchMode::TOPBOT),
+      m_multzMode(PinchMode::TOP),
+      m_pinchGapMode(PinchMode::GAP),
+      m_pinchMaxEmptyGap(ParserKeywords::PINCH::MAX_EMPTY_GAP::defaultValue)
+{}
 
 EclipseGrid::EclipseGrid(const std::array<int, 3>& dims ,
                          const std::vector<double>& coord ,
                          const std::vector<double>& zcorn ,
                          const int * actnum)
     : GridDims(dims),
-      m_minpvMode(MinpvMode::ModeEnum::Inactive),
-      m_pinchoutMode(PinchMode::ModeEnum::TOPBOT),
-      m_multzMode(PinchMode::ModeEnum::TOP),
-      m_pinchGapMode(PinchMode::ModeEnum::GAP)
+      m_minpvMode(MinpvMode::Inactive),
+      m_pinchoutMode(PinchMode::TOPBOT),
+      m_multzMode(PinchMode::TOP),
+      m_pinchGapMode(PinchMode::GAP),
+      m_pinchMaxEmptyGap(ParserKeywords::PINCH::MAX_EMPTY_GAP::defaultValue)
 {
     initCornerPointGrid( coord , zcorn , actnum );
 }
@@ -113,10 +121,11 @@ EclipseGrid::EclipseGrid(const std::array<int, 3>& dims ,
 
 EclipseGrid::EclipseGrid(const std::string& fileName )
     : GridDims(),
-      m_minpvMode(MinpvMode::ModeEnum::Inactive),
-      m_pinchoutMode(PinchMode::ModeEnum::TOPBOT),
-      m_multzMode(PinchMode::ModeEnum::TOP),
-      m_pinchGapMode(PinchMode::ModeEnum::GAP)
+      m_minpvMode(MinpvMode::Inactive),
+      m_pinchoutMode(PinchMode::TOPBOT),
+      m_multzMode(PinchMode::TOP),
+      m_pinchGapMode(PinchMode::GAP),
+      m_pinchMaxEmptyGap(ParserKeywords::PINCH::MAX_EMPTY_GAP::defaultValue)
 {
 
     Opm::EclIO::EclFile egridfile(fileName);
@@ -127,10 +136,11 @@ EclipseGrid::EclipseGrid(const std::string& fileName )
 
 EclipseGrid::EclipseGrid(const GridDims& gd)
     : GridDims(gd),
-      m_minpvMode(MinpvMode::ModeEnum::Inactive),
-      m_pinchoutMode(PinchMode::ModeEnum::TOPBOT),
-      m_multzMode(PinchMode::ModeEnum::TOP),
-      m_pinchGapMode(PinchMode::ModeEnum::GAP)
+      m_minpvMode(MinpvMode::Inactive),
+      m_pinchoutMode(PinchMode::TOPBOT),
+      m_multzMode(PinchMode::TOP),
+      m_pinchGapMode(PinchMode::GAP),
+      m_pinchMaxEmptyGap(ParserKeywords::PINCH::MAX_EMPTY_GAP::defaultValue)
 {
     this->m_nactive = this->getCartesianSize();
     this->active_volume = std::nullopt;
@@ -147,10 +157,11 @@ EclipseGrid::EclipseGrid(const GridDims& gd)
 EclipseGrid::EclipseGrid(size_t nx, size_t ny , size_t nz,
                          double dx, double dy, double dz)
     : GridDims(nx, ny, nz),
-      m_minpvMode(MinpvMode::ModeEnum::Inactive),
-      m_pinchoutMode(PinchMode::ModeEnum::TOPBOT),
-      m_multzMode(PinchMode::ModeEnum::TOP),
-      m_pinchGapMode(PinchMode::ModeEnum::GAP)
+      m_minpvMode(MinpvMode::Inactive),
+      m_pinchoutMode(PinchMode::TOPBOT),
+      m_multzMode(PinchMode::TOP),
+      m_pinchGapMode(PinchMode::GAP),
+      m_pinchMaxEmptyGap(ParserKeywords::PINCH::MAX_EMPTY_GAP::defaultValue)
 {
 
     m_coord.reserve((nx+1)*(ny+1)*6);
@@ -255,10 +266,11 @@ EclipseGrid::EclipseGrid(const EclipseGrid& src, const std::vector<int>& actnum)
 
 EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
     : GridDims(deck),
-      m_minpvMode(MinpvMode::ModeEnum::Inactive),
-      m_pinchoutMode(PinchMode::ModeEnum::TOPBOT),
-      m_multzMode(PinchMode::ModeEnum::TOP),
-      m_pinchGapMode(PinchMode::ModeEnum::GAP)
+      m_minpvMode(MinpvMode::Inactive),
+      m_pinchoutMode(PinchMode::TOPBOT),
+      m_multzMode(PinchMode::TOP),
+      m_pinchGapMode(PinchMode::GAP),
+      m_pinchMaxEmptyGap(ParserKeywords::PINCH::MAX_EMPTY_GAP::defaultValue)
 {
     if (deck.hasKeyword("GDFILE")){
 
@@ -303,6 +315,14 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
             apply_GRIDUNIT(deck.getActiveUnitSystem(), grid_units.value(), this->m_coord);
             if (this->m_rv.has_value())
                 apply_GRIDUNIT(deck.getActiveUnitSystem(), grid_units.value(), this->m_rv.value());
+
+            if (this->m_input_coord.has_value()) {
+                apply_GRIDUNIT(deck.getActiveUnitSystem(), grid_units.value(), this->m_input_coord.value());
+            }
+
+            if (this->m_input_zcorn.has_value()) {
+                apply_GRIDUNIT(deck.getActiveUnitSystem(), grid_units.value(), this->m_input_zcorn.value());
+            }
         }
     }
 }
@@ -336,36 +356,31 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
             m_pinch = item.getSIDouble(0);
 
             auto pinchoutString = record.getItem<ParserKeywords::PINCH::PINCHOUT_OPTION>().get< std::string >(0);
-            m_pinchoutMode = PinchMode::PinchModeFromString(pinchoutString);
+            m_pinchoutMode = PinchModeFromString(pinchoutString);
 
             auto multzString = record.getItem<ParserKeywords::PINCH::MULTZ_OPTION>().get< std::string >(0);
-            m_multzMode = PinchMode::PinchModeFromString(multzString);
+            m_multzMode = PinchModeFromString(multzString);
             auto pinchGapString = record.getItem<ParserKeywords::PINCH::CONTROL_OPTION>().get< std::string >(0);
-            m_pinchGapMode = PinchMode::PinchModeFromString(pinchGapString);
+            m_pinchGapMode = PinchModeFromString(pinchGapString);
+            m_pinchMaxEmptyGap = record.getItem<ParserKeywords::PINCH::MAX_EMPTY_GAP>().getSIDouble(0);
         }
 
-        if (deck.hasKeyword<ParserKeywords::MINPV>() && deck.hasKeyword<ParserKeywords::MINPVFIL>()) {
-            throw std::invalid_argument("Can not have both MINPV and MINPVFIL in the deck.");
-        } else if(deck.hasKeyword<ParserKeywords::MINPORV>() && deck.hasKeyword<ParserKeywords::MINPVFIL>()) {   
-            throw std::invalid_argument("Can not have both MINPORV and MINPVFIL in the deck.");
-        }
-        
         m_minpvVector.resize(getCartesianSize(), 0.0);
         if (deck.hasKeyword<ParserKeywords::MINPV>()) {
             const auto& record = deck.get<ParserKeywords::MINPV>( ).back().getRecord(0);
             const auto& item = record.getItem<ParserKeywords::MINPV::VALUE>( );
             std::fill(m_minpvVector.begin(), m_minpvVector.end(), item.getSIDouble(0));
-            m_minpvMode = MinpvMode::ModeEnum::EclSTD;
+            m_minpvMode = MinpvMode::EclSTD;
         } else if (deck.hasKeyword<ParserKeywords::MINPORV>()) {
             const auto& record = deck.get<ParserKeywords::MINPORV>( ).back().getRecord(0);
             const auto& item = record.getItem<ParserKeywords::MINPORV::VALUE>( );
             std::fill(m_minpvVector.begin(), m_minpvVector.end(), item.getSIDouble(0));
-            m_minpvMode = MinpvMode::ModeEnum::EclSTD;
+            m_minpvMode = MinpvMode::EclSTD;
         } else if(deck.hasKeyword<ParserKeywords::MINPVV>()) {
             // We should use the grid properties to support BOX, but then we need the eclipseState
             const auto& record = deck.get<ParserKeywords::MINPVV>( ).back().getRecord(0);
             m_minpvVector =record.getItem(0).getSIDoubleData();
-            m_minpvMode = MinpvMode::ModeEnum::EclSTD;
+            m_minpvMode = MinpvMode::EclSTD;
         }
 
         if (actnum != nullptr) {
@@ -521,22 +536,27 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
         return m_pinch.value();
     }
 
-    PinchMode::ModeEnum EclipseGrid::getPinchOption( ) const {
+    PinchMode EclipseGrid::getPinchOption( ) const {
         return m_pinchoutMode;
     }
 
-    PinchMode::ModeEnum EclipseGrid::getMultzOption( ) const {
+    PinchMode EclipseGrid::getMultzOption( ) const {
         return m_multzMode;
     }
 
-    MinpvMode::ModeEnum EclipseGrid::getMinpvMode() const {
+    MinpvMode EclipseGrid::getMinpvMode() const {
         return m_minpvMode;
     }
 
-    PinchMode::ModeEnum EclipseGrid::getPinchGapMode() const {
+    PinchMode EclipseGrid::getPinchGapMode() const {
         return m_pinchGapMode;
     }
 
+    double EclipseGrid::getPinchMaxEmptyGap() const {
+        return m_pinchMaxEmptyGap;
+    }
+
+
     const std::vector<double>& EclipseGrid::getMinpvVector( ) const {
         return m_minpvVector;
     }
@@ -1069,7 +1089,7 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
             CoordMapper cm(this->getNX(), this->getNY());
             std::vector<double> zcorn( zm.size() );
             std::vector<double> coord( cm.size() );
-            {                    
+            {
                 std::vector<double> depth(tops);
                 for (std::size_t k = 0; k < this->getNZ(); k++) {
                     for (std::size_t j = 0; j < this->getNY(); j++) {
@@ -1143,6 +1163,9 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
         m_coord = coord;
         m_zcorn = zcorn;
 
+        m_input_coord = coord;
+        m_input_zcorn = zcorn;
+
         ZcornMapper mapper( getNX(), getNY(), getNZ());
         zcorn_fixed = mapper.fixupZCORN( m_zcorn );
         this->resetACTNUM(actnum);
@@ -1235,7 +1258,7 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
         else
             return false;
     }
-    
+
     bool EclipseGrid::hasEqualDVDEPTHZ(const Deck& deck) {
         const std::vector<double>& DXV = deck.get<ParserKeywords::DXV>().back().getSIDoubleData();
         const std::vector<double>& DYV = deck.get<ParserKeywords::DYV>().back().getSIDoubleData();
@@ -1243,13 +1266,13 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
         const std::vector<double>& DEPTHZ = deck.get<ParserKeywords::DEPTHZ>().back().getSIDoubleData();
         if (EclipseGrid::allEqual(DXV) &&
             EclipseGrid::allEqual(DYV)&&
-            EclipseGrid::allEqual(DZV)&& 
+            EclipseGrid::allEqual(DZV)&&
             EclipseGrid::allEqual(DEPTHZ))
             return true;
         else
             return false;
     }
-    
+
     bool EclipseGrid::hasDTOPSKeywords(const Deck& deck) {
         if ((deck.hasKeyword<ParserKeywords::DX>() || deck.hasKeyword<ParserKeywords::DXV>()) &&
             (deck.hasKeyword<ParserKeywords::DY>() || deck.hasKeyword<ParserKeywords::DYV>()) &&
@@ -1380,12 +1403,12 @@ std::vector<double> EclipseGrid::createDVector(const std::array<int,3>& dims, st
             }
         }
     }
-    
+
     bool EclipseGrid::allEqual(const std::vector<double> &v) {
         auto comp = [](double x, double y) { return std::fabs(x - y) < 1e-12; };
         return std::equal(v.begin() + 1, v.end(), v.begin(), comp);
     }
-    
+
     bool EclipseGrid::equal(const EclipseGrid& other) const {
 
         //double reltol = 1.0e-6;
@@ -1410,7 +1433,7 @@ std::vector<double> EclipseGrid::createDVector(const std::array<int,3>& dims, st
 
         bool status = ((m_pinch == other.m_pinch)  && (m_minpvMode == other.getMinpvMode()));
 
-        if(m_minpvMode!=MinpvMode::ModeEnum::Inactive) {
+        if (m_minpvMode != MinpvMode::Inactive) {
             status = status && (m_minpvVector == other.getMinpvVector());
         }
 
@@ -1706,18 +1729,27 @@ std::vector<double> EclipseGrid::createDVector(const std::array<int,3>& dims, st
         std::vector<float> coord_f;
         coord_f.resize(m_coord.size());
 
-        for (size_t n=0; n< m_coord.size(); n++){
-            coord_f[n] = static_cast<double>(units.from_si(length, m_coord[n]));
+        auto convert_length = [&units](const double x) { return static_cast<float>(units.from_si(length, x)); };
+
+        if (m_input_coord.has_value()) {
+            std::transform(m_input_coord.value().begin(), m_input_coord.value().end(), coord_f.begin(), convert_length);
+        } else {
+            std::transform(m_coord.begin(), m_coord.end(), coord_f.begin(), convert_length);
         }
 
         // create zcorn vector of floats with input units, converted from SI
         std::vector<float> zcorn_f;
         zcorn_f.resize(m_zcorn.size());
 
-        for (size_t n=0; n< m_zcorn.size(); n++){
-            zcorn_f[n] = static_cast<double>(units.from_si(length, m_zcorn[n]));
+        if (m_input_coord.has_value()) {
+            std::transform(m_input_zcorn.value().begin(), m_input_zcorn.value().end(), zcorn_f.begin(), convert_length);
+        } else {
+            std::transform(m_zcorn.begin(), m_zcorn.end(), zcorn_f.begin(), convert_length);
         }
 
+        m_input_coord.reset();
+        m_input_zcorn.reset();
+
         std::vector<int> filehead(100,0);
         filehead[0] = 3;                     // version number
         filehead[1] = 2007;                  // release year
