@@ -154,8 +154,8 @@ void FieldPropsManager::apply_numerical_aquifers(const NumericalAquifers& aquife
     return this->fp->apply_numerical_aquifers(aquifers);
 }
 
-
-const Fieldprops::TranMap& FieldPropsManager::getTran() const
+const std::unordered_map<std::string,Fieldprops::TranCalculator>&
+FieldPropsManager::getTran() const
 {
     return this->fp->getTran();
 }
