@@ -35,6 +35,12 @@ BCType bctype(const std::string& s) {
     if (s == "FREE")
         return BCType::FREE;
 
+    if (s == "DIRICHLET")
+        return BCType::DIRICHLET;
+
+    if (s == "THERMAL")
+        return BCType::THERMAL;
+
     throw std::invalid_argument("Not recognized boundary condition type: " + s);
 }
 
@@ -64,6 +70,7 @@ BCComponent component(const std::string& s) {
 }
 }
 
+using BC = ParserKeywords::BC;
 BCConfig::BCFace::BCFace(const DeckRecord& record, const GridDims& grid) :
     i1(0),
     i2(grid.getNX() - 1),
@@ -71,12 +78,17 @@ BCConfig::BCFace::BCFace(const DeckRecord& record, const GridDims& grid) :
     j2(grid.getNY() - 1),
     k1(0),
     k2(grid.getNZ() - 1),
-    bctype(fromstring::bctype(record.getItem("TYPE").get<std::string>(0))),
-    dir(FaceDir::FromString(record.getItem("DIRECTION").get<std::string>(0))),
-    component(fromstring::component(record.getItem("COMPONENT").get<std::string>(0))),
-    rate(record.getItem("RATE").getSIDouble(0))
+    bctype(fromstring::bctype(record.getItem<BC::TYPE>().get<std::string>(0))),
+    dir(FaceDir::FromString(record.getItem<BC::DIRECTION>().get<std::string>(0))),
+    component(fromstring::component(record.getItem<BC::COMPONENT>().get<std::string>(0))),
+    rate(record.getItem<BC::RATE>().getSIDouble(0))
 {
-    using BC = ParserKeywords::BC;
+    if (const auto& P = record.getItem<BC::PRESSURE>(); ! P.defaultApplied(0)) {
+        pressure = P.getSIDouble(0);
+    }
+    if (const auto& T = record.getItem<BC::TEMPERATURE>(); ! T.defaultApplied(0)) {
+        temperature = T.getSIDouble(0);
+    }
     if (const auto& I1 = record.getItem<BC::I1>(); ! I1.defaultApplied(0)) {
         this->i1 = I1.get<int>(0) - 1;
     }
@@ -110,6 +122,8 @@ BCConfig::BCFace BCConfig::BCFace::serializationTestObject()
     result.dir = FaceDir::XPlus;
     result.component = BCComponent::GAS;
     result.rate = 100.0;
+    result.pressure = 101.0;
+    result.temperature = 102.0;
 
     return result;
 }
@@ -125,7 +139,9 @@ bool BCConfig::BCFace::operator==(const BCConfig::BCFace& other) const {
            this->bctype == other.bctype &&
            this->dir == other.dir &&
            this->component == other.component &&
-           this->rate == other.rate;
+           this->rate == other.rate &&
+           this->pressure == other.pressure &&
+           this->temperature == other.temperature;
 }
 
 
