@@ -30,7 +30,6 @@
 
 #include <cassert>
 #include <cmath>
-#include <iostream>
 #include <map>
 #include <iterator>
 #include <unordered_set>
@@ -73,6 +72,10 @@ namespace Opm {
         return m_segments.size();
     }
 
+    bool WellSegments::empty() const {
+        return this->m_segments.empty();
+    }
+
     const Segment& WellSegments::topSegment() const {
         return this->m_segments[0];
     }
