@@ -16,9 +16,9 @@
   You should have received a copy of the GNU General Public License
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
-#include <iostream>
-#include <string>
-#include <vector>
+
+#include <config.h>
+#include <opm/input/eclipse/Schedule/Well/Well.hpp>
 
 #include <opm/input/eclipse/Units/Units.hpp>
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
@@ -27,12 +27,13 @@
 #include <opm/input/eclipse/Schedule/SummaryState.hpp>
 #include <opm/input/eclipse/Schedule/ScheduleTypes.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQActive.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellInjectionProperties.hpp>
-#include <opm/input/eclipse/Schedule/Well/Well.hpp>
 
-#include "injection.hpp"
 #include "../eval_uda.hpp"
 
+#include <ostream>
+#include <string>
+#include <vector>
+
 namespace Opm {
 
 
@@ -48,18 +49,18 @@ namespace Opm {
           reservoirInjectionRate(units.getDimension(UnitSystem::measure::rate)),
           BHPTarget(units.getDimension(UnitSystem::measure::pressure)),
           THPTarget(units.getDimension(UnitSystem::measure::pressure)),
-          temperature(Metric::TemperatureOffset + ParserKeywords::STCOND::TEMPERATURE::defaultValue),
           BHPH(0),
           THPH(0),
           VFPTableNumber(0),
           predictionMode(true),
           injectionControls(0),
           injectorType(InjectorType::WATER),
-          controlMode(InjectorCMode::CMODE_UNDEFINED)
+          controlMode(InjectorCMode::CMODE_UNDEFINED),
+          rsRvInj(0.0)
     {
     }
 
-    Well::WellInjectionProperties Well::WellInjectionProperties::serializeObject()
+    Well::WellInjectionProperties Well::WellInjectionProperties::serializationTestObject()
     {
         Well::WellInjectionProperties result;
         result.name = "test";
@@ -69,7 +70,6 @@ namespace Opm {
         result.THPTarget = UDAValue(3.0);
         result.bhp_hist_limit = 4.0;
         result.thp_hist_limit = 5.0;
-        result.temperature = 6.0;
         result.BHPH = 7.0;
         result.THPH = 8.0;
         result.VFPTableNumber = 9;
@@ -77,6 +77,7 @@ namespace Opm {
         result.injectionControls = 10;
         result.injectorType = InjectorType::OIL;
         result.controlMode = InjectorCMode::BHP;
+        result.rsRvInj = 11;
 
         return result;
     }
@@ -123,13 +124,19 @@ namespace Opm {
             this->dropInjectionControl(InjectorCMode::GRUP);
         {
             const std::string& cmodeString = record.getItem("CMODE").getTrimmedString(0);
-            InjectorCMode controlModeArg = InjectorCModeFromString( cmodeString );
+            InjectorCMode controlModeArg = WellInjectorCModeFromString(cmodeString);
             if (this->hasInjectionControl( controlModeArg))
                 this->controlMode = controlModeArg;
             else {
                 throw std::invalid_argument("Tried to set invalid control: " + cmodeString + " for well: " + well_name);
             }
         }
+        this->rsRvInj = record.getItem("VAPOIL_C").getSIDouble(0);
+        if (this->injectorType == InjectorType::OIL && this->rsRvInj > 0) {
+            double factor = record.getItem("VAPOIL_C").getSIDouble(0) / record.getItem("VAPOIL_C").get<double>(0);
+            // for oil injectors the rsRvInj is Rs and the inverse of the si-conversion factor should be used
+            this->rsRvInj =  record.getItem("VAPOIL_C").get<double>(0) / factor;
+        }
     }
 
 
@@ -189,7 +196,7 @@ namespace Opm {
             this->THPH = record.getItem("THP").getSIDouble(0);
 
         const std::string& cmodeString = record.getItem("CMODE").getTrimmedString(0);
-        const InjectorCMode newControlMode = InjectorCModeFromString( cmodeString );
+        const InjectorCMode newControlMode = WellInjectorCModeFromString(cmodeString);
 
         if ( !(newControlMode == InjectorCMode::RATE || newControlMode == InjectorCMode::BHP) ) {
             const std::string msg = "Only RATE and BHP control are allowed for WCONINJH for well " + well_name;
@@ -220,12 +227,17 @@ namespace Opm {
         if (VFPTableNumberArg > 0) {
             this->VFPTableNumber = VFPTableNumberArg;
         }
+        this->rsRvInj = record.getItem("VAPOIL_C").getSIDouble(0);
+        if (this->injectorType == InjectorType::OIL && this->rsRvInj > 0) {
+            double factor = record.getItem("VAPOIL_C").getSIDouble(0) / record.getItem("VAPOIL_C").get<double>(0);
+            // for oil injectors the rsRvInj is Rs and the inverse of the si-conversion factor should be used
+            this->rsRvInj =  record.getItem("VAPOIL_C").get<double>(0) / factor;
+        }
     }
 
     bool Well::WellInjectionProperties::operator==(const Well::WellInjectionProperties& other) const {
         if ((surfaceInjectionRate == other.surfaceInjectionRate) &&
             (reservoirInjectionRate == other.reservoirInjectionRate) &&
-            (temperature == other.temperature) &&
             (BHPTarget == other.BHPTarget) &&
             (THPTarget == other.THPTarget) &&
             (BHPH == other.BHPH) &&
@@ -236,7 +248,8 @@ namespace Opm {
             (predictionMode == other.predictionMode) &&
             (injectionControls == other.injectionControls) &&
             (injectorType == other.injectorType) &&
-            (controlMode == other.controlMode))
+            (controlMode == other.controlMode) &&
+            (rsRvInj == other.rsRvInj))
             return true;
         else
             return false;
@@ -266,7 +279,6 @@ namespace Opm {
             << "Well::WellInjectionProperties { "
             << "surfacerate: "      << wp.surfaceInjectionRate << ", "
             << "reservoir rate "    << wp.reservoirInjectionRate << ", "
-            << "temperature: "      << wp.temperature << ", "
             << "BHP target: "       << wp.BHPTarget << ", "
             << "THP target: "       << wp.THPTarget << ", "
             << "BHPH: "             << wp.BHPH << ", "
@@ -275,7 +287,8 @@ namespace Opm {
             << "prediction mode: "  << wp.predictionMode << ", "
             << "injection ctrl: "   << wp.injectionControls << ", "
             << "injector type: "    << InjectorType2String(wp.injectorType) << ", "
-            << "control mode: "     << Well::InjectorCMode2String(wp.controlMode) << " }";
+            << "control mode: "     << WellInjectorCMode2String(wp.controlMode) << " , "
+            << "rs/rv concentration: " << wp.rsRvInj << " }";
     }
 
 
@@ -291,11 +304,11 @@ namespace Opm {
             controls.bhp_limit = this->bhp_hist_limit;
             controls.thp_limit = this->thp_hist_limit;
         }
-        controls.temperature = this->temperature;
         controls.injector_type = this->injectorType;
         controls.cmode = this->controlMode;
         controls.vfp_table_number = this->VFPTableNumber;
         controls.injector_type = this->injectorType;
+        controls.rs_rv_inj = this->rsRvInj;
 
         return controls;
     }
@@ -311,31 +324,101 @@ namespace Opm {
         return (update_count > 0);
     }
 
+    bool Well::WellInjectionProperties::updateUDQActive(const UDQConfig&   udq_config,
+                                                        const WELTARGCMode cmode,
+                                                        UDQActive&         active) const
+    {
+        switch (cmode) {
+        case WELTARGCMode::ORAT:
+            return (this->injectorType == InjectorType::OIL)
+                && (active.update(udq_config, this->surfaceInjectionRate, this->name, UDAControl::WELTARG_ORAT) > 0);
+
+        case WELTARGCMode::WRAT:
+            return (this->injectorType == InjectorType::WATER)
+                && (active.update(udq_config, this->surfaceInjectionRate, this->name, UDAControl::WELTARG_WRAT) > 0);
+
+        case WELTARGCMode::GRAT:
+            return (this->injectorType == InjectorType::GAS)
+                && (active.update(udq_config, this->surfaceInjectionRate, this->name, UDAControl::WELTARG_GRAT) > 0);
+
+        case WELTARGCMode::RESV:
+            return active.update(udq_config, this->reservoirInjectionRate, this->name, UDAControl::WELTARG_RESV) > 0;
+
+        case WELTARGCMode::BHP:
+            return active.update(udq_config, this->BHPTarget, this->name, UDAControl::WELTARG_BHP) > 0;
+
+        case WELTARGCMode::THP:
+            return active.update(udq_config, this->THPTarget, this->name, UDAControl::WELTARG_THP) > 0;
+
+        default:
+            return false;
+        }
+    }
+
+    void Well::WellInjectionProperties::update_uda(const UDQConfig& udq_config,
+                                                   UDQActive&       udq_active,
+                                                   const UDAControl control,
+                                                   const UDAValue&  value)
+    {
+        auto update_active = true;
 
-    void Well::WellInjectionProperties::update_uda(const UDQConfig& udq_config, UDQActive& udq_active, UDAControl control, const UDAValue& value) {
         switch (control) {
         case UDAControl::WCONINJE_RATE:
             this->surfaceInjectionRate = value;
-            udq_active.update(udq_config, this->surfaceInjectionRate, this->name, UDAControl::WCONINJE_RATE);
+            break;
+
+        case UDAControl::WELTARG_ORAT:
+            if (this->injectorType == InjectorType::OIL) {
+                this->surfaceInjectionRate = value;
+            }
+            else {
+                update_active = false;
+            }
+
+            break;
+
+        case UDAControl::WELTARG_WRAT:
+            if (this->injectorType == InjectorType::WATER) {
+                this->surfaceInjectionRate = value;
+            }
+            else {
+                update_active = false;
+            }
+            break;
+
+        case UDAControl::WELTARG_GRAT:
+            if (this->injectorType == InjectorType::GAS) {
+                this->surfaceInjectionRate = value;
+            }
+            else {
+                update_active = false;
+            }
             break;
 
         case UDAControl::WCONINJE_RESV:
+        case UDAControl::WELTARG_RESV:
             this->reservoirInjectionRate = value;
-            udq_active.update(udq_config, this->reservoirInjectionRate, this->name, UDAControl::WCONINJE_RESV);
             break;
 
         case UDAControl::WCONINJE_BHP:
+        case UDAControl::WELTARG_BHP:
             this->BHPTarget = value;
-            udq_active.update(udq_config, this->BHPTarget, this->name, UDAControl::WCONINJE_BHP);
             break;
 
         case UDAControl::WCONINJE_THP:
-            this->BHPTarget = value;
-            udq_active.update(udq_config, this->THPTarget, this->name, UDAControl::WCONINJE_THP);
+        case UDAControl::WELTARG_THP:
+            this->THPTarget = value;
             break;
 
         default:
-            throw std::logic_error("Invalid UDA control");
+            throw std::logic_error {
+                "Unsupported well injection UDA control '"
+                + UDQ::controlName(control) + '\''
+            };
+        }
+
+        if (update_active) {
+            udq_active.update(udq_config, value, this->name, control);
         }
     }
 
