@@ -23,8 +23,8 @@
 #include <opm/input/eclipse/Units/Dimension.hpp>
 #include <opm/input/eclipse/Units/Units.hpp>
 
+#include <algorithm>
 #include <cstddef>
-#include <iostream>
 #include <limits>
 #include <stdexcept>
 #include <vector>
@@ -91,6 +91,7 @@ namespace {
         0.0,
         0.0,
         0.0,
+        0.0,
     };
 
     static const double to_metric[] = {
@@ -135,6 +136,7 @@ namespace {
         1 / Metric::PolymerDensity,
         1 / Metric::Salinity,
         1 / (1 / Metric::Time),
+        1 / Metric::Moles,
     };
 
     static const double from_metric[] = {
@@ -179,6 +181,7 @@ namespace {
         Metric::PolymerDensity,
         Metric::Salinity,
         1 / Metric::Time,
+        Metric::Moles,
     };
 
     static constexpr const char* metric_names[static_cast<int>(UnitSystem::measure::_count)] = {
@@ -223,6 +226,7 @@ namespace {
         "KG / SM3", /*polymer density */
         "KG / SM3", /*salinity */
         "SM3/SM3/DAY",
+        "KG-M",
     };
 
     static_assert(numElems(from_metric_offset) == static_cast<std::size_t>(UnitSystem::measure::_count),
@@ -287,6 +291,7 @@ namespace {
         0.0,
         0.0,
         0.0,
+        0.0,
     };
 
     static const double to_field[] = {
@@ -331,6 +336,7 @@ namespace {
         1 / Field::PolymerDensity,
         1 / Field::Salinity,
         1 / (Field::GasSurfaceVolume / Field::LiquidSurfaceVolume / Field::Time),
+        1 / Field::Moles,
     };
 
     static const double from_field[] = {
@@ -375,6 +381,7 @@ namespace {
          Field::PolymerDensity,
          Field::Salinity,
          Field::GasSurfaceVolume / Field::LiquidSurfaceVolume / Field::Time,
+         Field::Moles,
     };
 
     static constexpr const char* field_names[static_cast<int>(UnitSystem::measure::_count)] = {
@@ -419,6 +426,7 @@ namespace {
         "LB/STB", /*polymer density */
         "LB/STB", /*salinity */
         "MSCF/STB/DAY",
+        "LB-M",
     };
 
     static_assert(numElems(from_field_offset) == static_cast<std::size_t>(UnitSystem::measure::_count),
@@ -483,6 +491,7 @@ namespace {
         0.0,
         0.0,
         0.0,
+        0.0,
     };
 
     static const double to_lab[] = {
@@ -527,6 +536,7 @@ namespace {
         1 / Lab::PolymerDensity,
         1 / Lab::Salinity,
         1 / (Lab::GasDissolutionFactor / Lab::Time),
+        1 / Lab::Moles,
     };
 
     static const double from_lab[] = {
@@ -571,6 +581,7 @@ namespace {
         Lab::PolymerDensity,
         Lab::Salinity,
         Lab::GasDissolutionFactor / Lab::Time,
+        Lab::Moles,
     };
 
     static constexpr const char* lab_names[static_cast<int>(UnitSystem::measure::_count)] = {
@@ -615,6 +626,7 @@ namespace {
         "G/SCC", /*polymer density */
         "G/SCC", /*salinity */
         "SCC/SCC/HR",
+        "g-M",
     };
 
     static_assert(numElems(from_lab_offset) == static_cast<std::size_t>(UnitSystem::measure::_count),
@@ -679,6 +691,7 @@ namespace {
         0.0,
         0.0,
         0.0,
+        0.0,
     };
 
     static const double to_pvt_m[] = {
@@ -723,6 +736,7 @@ namespace {
         1 / PVT_M::PolymerDensity,
         1 / PVT_M::Salinity,
         1 / (PVT_M::GasSurfaceVolume / PVT_M::LiquidSurfaceVolume /PVT_M::Time),
+        1 / PVT_M::Moles,
     };
 
     static const double from_pvt_m[] = {
@@ -767,6 +781,7 @@ namespace {
         PVT_M::PolymerDensity,
         PVT_M::Salinity,
         PVT_M::GasSurfaceVolume / PVT_M::LiquidSurfaceVolume /PVT_M::Time,
+        PVT_M::Moles,
     };
 
     static constexpr const char* pvt_m_names[static_cast<int>(UnitSystem::measure::_count)] = {
@@ -811,6 +826,7 @@ namespace {
         "KG/SM3", /*polymer density */
         "KG/SM3", /*salinity */
         "SM3/SM3/DAY",
+        "KG-M",
     };
 
     static_assert(numElems(from_pvt_m_offset) == static_cast<std::size_t>(UnitSystem::measure::_count),
@@ -875,6 +891,7 @@ namespace {
         0.0,
         0.0,
         0.0,
+        0.0,
     };
 
     static const double to_input[] = {
@@ -919,6 +936,7 @@ namespace {
         1,
         1,
         1,
+        1,
     };
 
     static const double from_input[] = {
@@ -963,6 +981,7 @@ namespace {
         1,
         1,
         1,
+        1,
     };
 
     static constexpr const char* input_names[static_cast<int>(UnitSystem::measure::_count)] = {
@@ -1007,6 +1026,7 @@ namespace {
         "KG/SM3", /*polymer density */
         "KG/SM3", /*salinity */
         "SM3/SM3/DAY",
+        "g-M"
     };
 
     static_assert(numElems(from_input_offset) == static_cast<std::size_t>(UnitSystem::measure::_count),
