@@ -2346,6 +2346,34 @@ BOOST_AUTO_TEST_CASE( TestParseDIFFC ) {
     BOOST_CHECK_CLOSE( 1.8, diffc[0].oil_in_oil_cross_phase*conversion_factor, epsilon() );
 }
 
+
+BOOST_AUTO_TEST_CASE( TestParseDIFFCWATGAS ) {
+    const std::string data = R"(
+      TABDIMS
+        1* 1 /
+
+      DIFFCWAT
+        1.1 1.2 /
+
+      DIFFCGAS
+        1.3 1.4 /
+    )";
+
+    Opm::Parser parser;
+    auto deck = parser.parseString(data);
+    Opm::TableManager tables( deck );
+    double conversion_factor = (60*60*24);
+
+    const auto& diffcwat = tables.getDiffusionCoefficientWaterTable();
+    BOOST_CHECK_CLOSE( 1.1, diffcwat[0].co2_in_water*conversion_factor, epsilon() );
+    BOOST_CHECK_CLOSE( 1.2, diffcwat[0].h2o_in_water*conversion_factor, epsilon());
+
+    const auto& diffcgas = tables.getDiffusionCoefficientGasTable();
+    BOOST_CHECK_CLOSE( 1.3, diffcgas[0].co2_in_gas*conversion_factor, epsilon() );
+    BOOST_CHECK_CLOSE( 1.4, diffcgas[0].h2o_in_gas*conversion_factor, epsilon() );
+}
+
+
 BOOST_AUTO_TEST_CASE( TestParseROCK ) {
     const std::string data = R"(
       TABDIMS
