@@ -1,14 +1,18 @@
 /*
-Copyright 2018 Statoil ASA.
+  Copyright 2018 Statoil ASA.
+
   This file is part of the Open Porous Media project (OPM).
+
   OPM is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
+
   OPM is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
+
   You should have received a copy of the GNU General Public License
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
@@ -43,6 +47,7 @@ Copyright 2018 Statoil ASA.
 #include <opm/input/eclipse/Utility/Typetools.hpp>
 
 #include <opm/input/eclipse/Parser/ErrorGuard.hpp>
+#include <opm/input/eclipse/Parser/InputErrorAction.hpp>
 #include <opm/input/eclipse/Parser/ParseContext.hpp>
 #include <opm/input/eclipse/Parser/Parser.hpp>
 
@@ -91,7 +96,6 @@ BOOST_AUTO_TEST_CASE(TYPE_COERCION) {
     BOOST_CHECK_THROW( UDQ::coerce(UDQVarType::WELL_VAR, UDQVarType::GROUP_VAR), std::logic_error );
 }
 
-
 BOOST_AUTO_TEST_CASE(GROUP_VARIABLES)
 {
     KeywordLocation location;
@@ -128,7 +132,8 @@ BOOST_AUTO_TEST_CASE(SUBTRACT)
 
     SummaryState st(TimeService::now());
     UDQState udq_state(udqp.undefinedValue());
-    UDQContext context(udqft, WellMatcher(NameOrder({"P1"})), st, udq_state);
+    WellMatcher wm(NameOrder({"P1"}));
+    UDQContext context(udqft, wm, st, udq_state);
 
     st.update_well_var("P1", "WOPR", 4);
     auto res = def.eval(context);
@@ -150,7 +155,8 @@ BOOST_AUTO_TEST_CASE(TEST)
 
     SummaryState st(TimeService::now());
     UDQState udq_state(udqp.undefinedValue());
-    UDQContext context(udqft, WellMatcher(NameOrder({"P1", "P2"})), st, udq_state);
+    WellMatcher wm(NameOrder({"P1", "P2"}));
+    UDQContext context(udqft, wm, st, udq_state);
 
     st.update_group_var("MAU", "GOPR", 4);
     st.update_group_var("XXX", "GOPR", 5);
@@ -175,19 +181,14 @@ BOOST_AUTO_TEST_CASE(TEST)
     BOOST_CHECK_EQUAL( res4["P1"].get(), 1.00 );
     BOOST_CHECK_EQUAL( res4["P2"].get(), 1.00 );
 
-    /*
-      This expression has a well set as target type, and involves group with
-      wildcard that is not supported by flow.
-    */
-    BOOST_CHECK_THROW( UDQDefine(udqp, "WUWI2",0, location, {"GOPR", "G*", "*", "2.0"}), OpmInputError);
+    // This expression has a well set as target type, and involves group
+    // with wildcard that is not supported by flow.
+    BOOST_CHECK_THROW( UDQDefine(udqp, "GUPR2",0, location, {"GOPR", "G*", "*", "2.0"}), OpmInputError);
 
-    /*
-      UDQVarType == BLOCK is not yet supported.
-    */
-    BOOST_CHECK_THROW( UDQDefine(udqp, "WUWI2",0, location, {"BPR", "1","1", "1", "*", "2.0"}), OpmInputError);
+    // UDQVarType == BLOCK is not yet supported.
+    BOOST_CHECK_THROW( UDQDefine(udqp, "BUPR2",0, location, {"BPR", "1","1", "1", "*", "2.0"}), std::invalid_argument);
 }
 
-
 BOOST_AUTO_TEST_CASE(MIX_SCALAR) {
     UDQFunctionTable udqft;
     UDQParams udqp;
@@ -195,7 +196,8 @@ BOOST_AUTO_TEST_CASE(MIX_SCALAR) {
     UDQDefine def_add(udqp, "WU",0, location, {"WOPR", "+", "1"});
     SummaryState st(TimeService::now());
     UDQState udq_state(udqp.undefinedValue());
-    UDQContext context(udqft, WellMatcher(NameOrder({"P1"})), st, udq_state);
+    WellMatcher wm(NameOrder({"P1"}));
+    UDQContext context(udqft, wm, st, udq_state);
 
     st.update_well_var("P1", "WOPR", 1);
 
@@ -203,15 +205,12 @@ BOOST_AUTO_TEST_CASE(MIX_SCALAR) {
     BOOST_CHECK_EQUAL( res_add["P1"].get() , 2);
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQ_TABLE_EXCEPTION) {
     UDQParams udqp;
     KeywordLocation location;
     BOOST_CHECK_THROW(UDQDefine(udqp, "WU",0, location, {"TUPRICE[WOPR]"}), std::invalid_argument);
 }
 
-
-
 BOOST_AUTO_TEST_CASE(UDQFieldSetTest) {
     std::vector<std::string> wells = {"P1", "P2", "P3", "P4"};
     KeywordLocation location;
@@ -219,20 +218,22 @@ BOOST_AUTO_TEST_CASE(UDQFieldSetTest) {
     UDQFunctionTable udqft(udqp);
     SummaryState st(TimeService::now());
     UDQState udq_state(udqp.undefinedValue());
-    UDQContext context(udqft, WellMatcher(NameOrder({"P1", "P2", "P3", "P4"})), st, udq_state);
+    WellMatcher wm(NameOrder({"P1", "P2", "P3", "P4"}));
+    UDQContext context(udqft, wm, st, udq_state);
 
     st.update_well_var("P1", "WOPR", 1.0);
     st.update_well_var("P2", "WOPR", 2.0);
     st.update_well_var("P3", "WOPR", 3.0);
     st.update_well_var("P4", "WOPR", 4.0);
 
-    /*{
+#if 0
+    {
         UDQDefine def_fxxx(udqp, "FU_SCALAR", {"123"});
         auto fxxx_res = def_fxxx.eval(context);
         BOOST_CHECK_EQUAL( fxxx_res[0].get(), 123.0 );
         BOOST_CHECK( fxxx_res.var_type() == UDQVarType::FIELD_VAR);
     }
-    */
+#endif
 
     {
         UDQDefine def_fopr(udqp, "FUOPR",0, location, {"SUM", "(", "WOPR", ")"});
@@ -261,7 +262,6 @@ BOOST_AUTO_TEST_CASE(UDQWellSetNANTest) {
     BOOST_CHECK(ws.has("P2"));
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQWellSetTest) {
     std::vector<std::string> wells = {"P1", "P2", "I1", "I2"};
     UDQSet ws = UDQSet::wells("NAME", wells);
@@ -305,7 +305,6 @@ BOOST_AUTO_TEST_CASE(UDQWellSetTest) {
     BOOST_CHECK_EQUAL(empty.size() , 0U);
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQ_GROUP_TEST) {
     std::vector<std::string> groups = {"G1", "G2", "G3", "G4"};
     UDQSet gs = UDQSet::groups("NAME", groups);
@@ -335,8 +334,6 @@ BOOST_AUTO_TEST_CASE(UDQ_GROUP_TEST) {
     }
 }
 
-
-
 BOOST_AUTO_TEST_CASE(UDQ_DEFINETEST) {
     UDQParams udqp;
     UDQFunctionTable udqft(udqp);
@@ -345,7 +342,8 @@ BOOST_AUTO_TEST_CASE(UDQ_DEFINETEST) {
         UDQDefine def(udqp, "WUBHP",0, location, {"WBHP"});
         SummaryState st(TimeService::now());
         UDQState udq_state(udqp.undefinedValue());
-        UDQContext context(udqft, WellMatcher(NameOrder({"W1", "W2", "W3"})), st, udq_state);
+        WellMatcher wm(NameOrder({"W1", "W2", "W3"}));
+        UDQContext context(udqft, wm, st, udq_state);
 
         st.update_well_var("W1", "WBHP", 11);
         st.update_well_var("W2", "WBHP", 2);
@@ -364,7 +362,8 @@ BOOST_AUTO_TEST_CASE(UDQ_DEFINETEST) {
         UDQDefine def(udqp, "WUBHP",0, location, {"WBHP" , "'P*'"});
         SummaryState st(TimeService::now());
         UDQState udq_state(udqp.undefinedValue());
-        UDQContext context(udqft, WellMatcher(NameOrder({"I1", "I2", "P1", "P2"})), st, udq_state);
+        WellMatcher wm(NameOrder({"I1", "I2", "P1", "P2"}));
+        UDQContext context(udqft, wm, st, udq_state);
 
 
         st.update_well_var("P1", "WBHP", 1);
@@ -382,7 +381,8 @@ BOOST_AUTO_TEST_CASE(UDQ_DEFINETEST) {
         UDQDefine def(udqp, "WUBHP",0, location, {"NINT" , "(", "WBHP", ")"});
         SummaryState st(TimeService::now());
         UDQState udq_state(udqp.undefinedValue());
-        UDQContext context(udqft, WellMatcher(NameOrder({"P1", "P2", "I1", "I2"})), st, udq_state);
+        WellMatcher wm(NameOrder({"P1", "P2", "I1", "I2"}));
+        UDQContext context(udqft, wm, st, udq_state);
         st.update_well_var("P1", "WBHP", 4);
         st.update_well_var("P2", "WBHP", 3);
         st.update_well_var("I1", "WBHP", 2);
@@ -396,10 +396,6 @@ BOOST_AUTO_TEST_CASE(UDQ_DEFINETEST) {
     }
 }
 
-
-
-
-
 BOOST_AUTO_TEST_CASE(KEYWORDS) {
     const std::string input = R"(
 RUNSPEC
@@ -427,7 +423,6 @@ UDQPARAM
     BOOST_CHECK( r1 != r2 );
 }
 
-
 BOOST_AUTO_TEST_CASE(ENUM_CONVERSION) {
     BOOST_CHECK_THROW(UDQ::varType("WWCT"), std::invalid_argument);
     BOOST_CHECK_THROW(UDQ::varType("XUCT"), std::invalid_argument);
@@ -454,7 +449,6 @@ BOOST_AUTO_TEST_CASE(ENUM_CONVERSION) {
     BOOST_CHECK(UDQ::actionType("ASSIGN") == UDQAction::ASSIGN );
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQ_KEWYORDS) {
     const std::string input = R"(
 RUNSPEC
@@ -536,10 +530,6 @@ UDQ
   BOOST_CHECK_THROW( make_schedule(input), std::exception);
 }
 
-
-
-
-
 BOOST_AUTO_TEST_CASE(UDQ_DEFINE_WITH_SLASH) {
     const std::string input = R"(
 UDQ
@@ -557,7 +547,6 @@ UDQ
                                   exp.begin(), exp.end());
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQ_ASSIGN_DATA) {
     const std::string input = R"(
 RUNSPEC
@@ -663,7 +652,6 @@ BOOST_AUTO_TEST_CASE(UDQ_SET) {
     }
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQ_FUNCTION_TABLE) {
     UDQFunctionTable udqft;
     BOOST_CHECK(udqft.has_function("SUM"));
@@ -812,7 +800,6 @@ BOOST_AUTO_TEST_CASE(CMP_FUNCTIONS) {
     }
 }
 
-
 BOOST_AUTO_TEST_CASE(CMP_FUNCTIONS2) {
     UDQFunctionTable udqft;
     auto arg1 = UDQSet::scalar("NAME", 0);
@@ -822,7 +809,6 @@ BOOST_AUTO_TEST_CASE(CMP_FUNCTIONS2) {
     BOOST_CHECK_EQUAL(eq[0].get(), 1);
 }
 
-
 BOOST_AUTO_TEST_CASE(ELEMENTAL_UNARY_FUNCTIONS) {
     UDQFunctionTable udqft;
     UDQSet arg("NAME", 5);
@@ -930,7 +916,6 @@ BOOST_AUTO_TEST_CASE(ELEMENTAL_UNARY_FUNCTIONS) {
     }
 }
 
-
 BOOST_AUTO_TEST_CASE(UNION_FUNCTIONS) {
     UDQFunctionTable udqft;
     UDQSet arg1("NAME", 5);
@@ -950,9 +935,6 @@ BOOST_AUTO_TEST_CASE(UNION_FUNCTIONS) {
     BOOST_CHECK_EQUAL( 3, result[3].get() );
 }
 
-
-
-
 BOOST_AUTO_TEST_CASE(FUNCTIONS_INVALID_ARGUMENT) {
     UDQSet arg("NAME",3);
     arg.assign(0, -1);
@@ -975,8 +957,6 @@ BOOST_AUTO_TEST_CASE(UDQ_SET_DIV) {
     BOOST_CHECK_EQUAL( result[4].get(), 2);
 }
 
-
-
 BOOST_AUTO_TEST_CASE(UDQASSIGN_TEST) {
     UDQAssign as1("WUPR", std::vector<std::string>{}, 1.0, 1);
     UDQAssign as2("WUPR", std::vector<std::string>{"P*"}, 2.0, 2);
@@ -1001,6 +981,59 @@ BOOST_AUTO_TEST_CASE(UDQASSIGN_TEST) {
     BOOST_CHECK(!res3["I2"].defined());
 }
 
+BOOST_AUTO_TEST_CASE(UDQASSIGN_NUMBERED_ITEMS_TEST) {
+    using Vsz = std::vector<std::size_t>;
+
+    const auto selector = UDQSet::EnumeratedWellItems {
+        "PROD01", Vsz { 1, 3, 5, 7 }
+    };
+
+    const auto as = UDQAssign {
+        "SUVTRIG", std::vector { selector }, 0.123, 42
+    };
+
+    const auto segments = std::vector {
+        UDQSet::EnumeratedWellItems { "PROD01", Vsz { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, } },
+        UDQSet::EnumeratedWellItems { "PROD02", Vsz { 1, 2, 3, 4, 5, } },
+        UDQSet::EnumeratedWellItems { "PROD06", Vsz { 2, 4, 6, 8, 10, } },
+        UDQSet::EnumeratedWellItems { "I-45", Vsz { 1, 2, 3, } },
+    };
+
+    const auto us = as.eval(segments);
+
+    BOOST_CHECK_EQUAL(us.size(), std::size_t{23});
+    BOOST_CHECK_CLOSE(us("PROD01", 1).get(), 0.123, 1.0e-8);
+    BOOST_CHECK_MESSAGE(!us("PROD01", 2).defined(), R"(SUVTRIG("PROD01", 2) must not be defined)");
+    BOOST_CHECK_CLOSE(us("PROD01", 3).get(), 0.123, 1.0e-8);
+    BOOST_CHECK_MESSAGE(!us("PROD01", 4).defined(), R"(SUVTRIG("PROD01", 4) must not be defined)");
+    BOOST_CHECK_CLOSE(us("PROD01", 5).get(), 0.123, 1.0e-8);
+    BOOST_CHECK_MESSAGE(!us("PROD01", 6).defined(), R"(SUVTRIG("PROD01", 6) must not be defined)");
+    BOOST_CHECK_CLOSE(us("PROD01", 7).get(), 0.123, 1.0e-8);
+    BOOST_CHECK_MESSAGE(!us("PROD01",  8).defined(), R"(SUVTRIG("PROD01", 8) must not be defined)");
+    BOOST_CHECK_MESSAGE(!us("PROD01",  9).defined(), R"(SUVTRIG("PROD01", 9) must not be defined)");
+    BOOST_CHECK_MESSAGE(!us("PROD01", 10).defined(), R"(SUVTRIG("PROD01", 10) must not be defined)");
+
+    BOOST_CHECK_MESSAGE(!us("PROD02", 1).defined(), R"(SUVTRIG("PROD02", 1) must not be defined)");
+    BOOST_CHECK_MESSAGE(!us("PROD02", 2).defined(), R"(SUVTRIG("PROD02", 2) must not be defined)");
+    BOOST_CHECK_MESSAGE(!us("PROD02", 3).defined(), R"(SUVTRIG("PROD02", 3) must not be defined)");
+    BOOST_CHECK_MESSAGE(!us("PROD02", 4).defined(), R"(SUVTRIG("PROD02", 4) must not be defined)");
+    BOOST_CHECK_MESSAGE(!us("PROD02", 5).defined(), R"(SUVTRIG("PROD02", 5) must not be defined)");
+
+    BOOST_CHECK_MESSAGE(!us("PROD06",  2).defined(), R"(SUVTRIG("PROD06", 2) must not be defined)");
+    BOOST_CHECK_MESSAGE(!us("PROD06",  4).defined(), R"(SUVTRIG("PROD06", 4) must not be defined)");
+    BOOST_CHECK_MESSAGE(!us("PROD06",  6).defined(), R"(SUVTRIG("PROD06", 6) must not be defined)");
+    BOOST_CHECK_MESSAGE(!us("PROD06",  8).defined(), R"(SUVTRIG("PROD06", 8) must not be defined)");
+    BOOST_CHECK_MESSAGE(!us("PROD06", 10).defined(), R"(SUVTRIG("PROD06", 10) must not be defined)");
+
+    BOOST_CHECK_THROW(!us("PROD06", 3).defined(), std::out_of_range);
+
+    BOOST_CHECK_MESSAGE(!us("I-45", 1).defined(), R"(SUVTRIG("I-45", 1) must not be defined)");
+    BOOST_CHECK_MESSAGE(!us("I-45", 2).defined(), R"(SUVTRIG("I-45", 2) must not be defined)");
+    BOOST_CHECK_MESSAGE(!us("I-45", 3).defined(), R"(SUVTRIG("I-45", 3) must not be defined)");
+
+    BOOST_CHECK_THROW(!us("Hello", 42).defined(), std::out_of_range);
+}
+
 BOOST_AUTO_TEST_CASE(UDQ_POW_TEST) {
     KeywordLocation location;
     UDQFunctionTable udqft;
@@ -1010,7 +1043,8 @@ BOOST_AUTO_TEST_CASE(UDQ_POW_TEST) {
     SummaryState st(TimeService::now());
     UDQState udq_state(udqp.undefinedValue());
     NameOrder wo; wo.add("P1");
-    UDQContext context(udqft, WellMatcher(wo), st, udq_state);
+    WellMatcher wm(wo);
+    UDQContext context(udqft, wm, st, udq_state);
 
     st.update_well_var("P1", "WOPR", 1);
     st.update_well_var("P1", "WWPR", 2);
@@ -1031,7 +1065,8 @@ BOOST_AUTO_TEST_CASE(UDQ_CMP_TEST) {
     UDQDefine def_cmp(udqp, "WU",0, location, {"WOPR", ">", "WWPR", "+", "WGOR", "*", "WWIR"});
     SummaryState st(TimeService::now());
     UDQState udq_state(udqp.undefinedValue());
-    UDQContext context(udqft, WellMatcher(NameOrder({"P1", "P2"})), st, udq_state);
+    WellMatcher wm(NameOrder({"P1", "P2"}));
+    UDQContext context(udqft, wm, st, udq_state);
 
     st.update_well_var("P1", "WOPR",  0);
     st.update_well_var("P1", "WWPR", 10);
@@ -1048,11 +1083,12 @@ BOOST_AUTO_TEST_CASE(UDQ_CMP_TEST) {
     BOOST_CHECK_EQUAL( res_cmp["P2"].get() , 0.0);
 }
 
-/*BOOST_AUTO_TEST_CASE(UDQPARSE_ERROR) {
+#if 0
+BOOST_AUTO_TEST_CASE(UDQPARSE_ERROR) {
     setUDQFunctionTable udqft;
     UDQDefine def1(udqft, "WUBHP", {"WWCT", "+"});
 }
-*/
+#endif
 
 BOOST_AUTO_TEST_CASE(UDQ_SCALAR_SET) {
     KeywordLocation location;
@@ -1060,7 +1096,8 @@ BOOST_AUTO_TEST_CASE(UDQ_SCALAR_SET) {
     UDQFunctionTable udqft;
     SummaryState st(TimeService::now());
     UDQState udq_state(udqp.undefinedValue());
-    UDQContext context(udqft, WellMatcher(NameOrder({"PA1", "PB2", "PC3", "PD4"})), st, udq_state);
+    WellMatcher wm(NameOrder({"PA1", "PB2", "PC3", "PD4"}));
+    UDQContext context(udqft, wm, st, udq_state);
 
     st.update_well_var("PA1", "WOPR", 1);
     st.update_well_var("PB2", "WOPR", 2);
@@ -1130,7 +1167,8 @@ BOOST_AUTO_TEST_CASE(UDQ_SORTD_NAN) {
     UDQDefine def_sort(udqp , "WUPR3",0, location, {"SORTD", "(", "WUPR1", ")" });
     SummaryState st(TimeService::now());
     UDQState udq_state(udqp.undefinedValue());
-    UDQContext context(udqft, WellMatcher(NameOrder({"OP1", "OP2", "OP3", "OP4"})), st, udq_state);
+    WellMatcher wm(NameOrder({"OP1", "OP2", "OP3", "OP4"}));
+    UDQContext context(udqft, wm, st, udq_state);
 
     st.update_well_var("OP1", "WWIR", 1.0);
     st.update_well_var("OP2", "WWIR", 2.0);
@@ -1166,8 +1204,6 @@ BOOST_AUTO_TEST_CASE(UDQ_SORTD_NAN) {
     BOOST_CHECK( st.has_well_var("OP4", "WUPR3"));
 }
 
-
-
 BOOST_AUTO_TEST_CASE(UDQ_SORTA) {
     KeywordLocation location;
     UDQParams udqp;
@@ -1176,7 +1212,8 @@ BOOST_AUTO_TEST_CASE(UDQ_SORTA) {
     UDQDefine def_sort(udqp , "WUPR3",0, location, {"SORTA", "(", "WUPR1", ")" });
     SummaryState st(TimeService::now());
     UDQState udq_state(udqp.undefinedValue());
-    UDQContext context(udqft, WellMatcher(NameOrder({"OPL01", "OPL02", "OPU01", "OPU02"})), st, udq_state);
+    WellMatcher wm(NameOrder({"OPL01", "OPL02", "OPU01", "OPU02"}));
+    UDQContext context(udqft, wm, st, udq_state);
 
     st.update_well_var("OPL01", "WWCT", 0.7);
     st.update_well_var("OPL02", "WWCT", 0.8);
@@ -1192,8 +1229,6 @@ BOOST_AUTO_TEST_CASE(UDQ_SORTA) {
     BOOST_CHECK_EQUAL(res_sort["OPU01"].get() + res_sort["OPU02"].get(), 7.0);
 }
 
-
-
 BOOST_AUTO_TEST_CASE(UDQ_BASIC_MATH_TEST) {
     UDQParams udqp;
     UDQFunctionTable udqft;
@@ -1206,7 +1241,8 @@ BOOST_AUTO_TEST_CASE(UDQ_BASIC_MATH_TEST) {
     UDQDefine def_wuwct(udqp , "WUWCT",0, location, {"WWPR", "/", "(", "WOPR", "+", "WWPR", ")"});
     SummaryState st(TimeService::now());
     UDQState udq_state(udqp.undefinedValue());
-    UDQContext context(udqft, WellMatcher(NameOrder({"P1", "P2", "P3", "P4"})), st, udq_state);
+    WellMatcher wm(NameOrder({"P1", "P2", "P3", "P4"}));
+    UDQContext context(udqft, wm, st, udq_state);
 
     st.update_well_var("P1", "WOPR", 1);
     st.update_well_var("P2", "WOPR", 2);
@@ -1268,7 +1304,8 @@ BOOST_AUTO_TEST_CASE(DECK_TEST) {
     UDQDefine def(udqp, "WUOPRL",0, location, {"(", "WOPR", "OP1", "-", "150", ")", "*", "0.90"});
     SummaryState st(TimeService::now());
     UDQState udq_state(udqp.undefinedValue());
-    UDQContext context(udqft, WellMatcher(NameOrder({"OP1", "OP2", "OP3"})), st, udq_state);
+    WellMatcher wm(NameOrder({"OP1", "OP2", "OP3"}));
+    UDQContext context(udqft, wm, st, udq_state);
 
     st.update_well_var("OP1", "WOPR", 300);
     st.update_well_var("OP2", "WOPR", 3000);
@@ -1280,7 +1317,6 @@ BOOST_AUTO_TEST_CASE(DECK_TEST) {
         BOOST_CHECK( res[index].get() == (300 - 150)*0.90);
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQPARSE_TEST1) {
     KeywordLocation location;
     UDQParams udqp;
@@ -1317,20 +1353,21 @@ BOOST_AUTO_TEST_CASE(UDQ_PARSE_ERROR) {
     ErrorGuard errors;
     std::vector<std::string> tokens = {"WBHP", "+"};
     KeywordLocation location;
-    parseContext.update(ParseContext::UDQ_PARSE_ERROR, InputError::IGNORE);
+    parseContext.update(ParseContext::UDQ_PARSE_ERROR, InputErrorAction::IGNORE);
     {
         UDQDefine def1(udqp, "WUBHP",0, location, tokens, parseContext, errors);
         SummaryState st(TimeService::now());
         UDQFunctionTable udqft(udqp);
         UDQState udq_state(udqp.undefinedValue());
-        UDQContext context(udqft, WellMatcher(NameOrder({"P1"})), st, udq_state);
+        WellMatcher wm(NameOrder({"P1"}));
+        UDQContext context(udqft, wm, st, udq_state);
         st.update_well_var("P1", "WBHP", 1);
 
         auto res = def1.eval(context);
         BOOST_CHECK_EQUAL(res["P1"].get(), udqp.undefinedValue());
     }
 
-    parseContext.update(ParseContext::UDQ_PARSE_ERROR, InputError::THROW_EXCEPTION);
+    parseContext.update(ParseContext::UDQ_PARSE_ERROR, InputErrorAction::THROW_EXCEPTION);
     BOOST_CHECK_THROW( UDQDefine(udqp, "WUBHP",0, location, tokens, parseContext, errors), OpmInputError);
 }
 
@@ -1341,7 +1378,7 @@ BOOST_AUTO_TEST_CASE(UDQ_TYPE_ERROR) {
     std::vector<std::string> tokens1 = {"WBHP", "+", "1"};
     std::vector<std::string> tokens2 = {"SUM", "(", "WBHP", ")"};
     KeywordLocation location;
-    parseContext.update(ParseContext::UDQ_TYPE_ERROR, InputError::IGNORE);
+    parseContext.update(ParseContext::UDQ_TYPE_ERROR, InputErrorAction::IGNORE);
     {
         UDQDefine def1(udqp, "FUBHP",0, location, tokens1, parseContext, errors);
         UDQDefine def2(udqp, "WUBHP",0, location, tokens2, parseContext, errors);
@@ -1349,7 +1386,8 @@ BOOST_AUTO_TEST_CASE(UDQ_TYPE_ERROR) {
         SummaryState st(TimeService::now());
         UDQFunctionTable udqft(udqp);
         UDQState udq_state(udqp.undefinedValue());
-        UDQContext context(udqft, WellMatcher(NameOrder({"P1", "P2"})), st, udq_state);
+        WellMatcher wm(NameOrder({"P1", "P2"}));
+        UDQContext context(udqft, wm, st, udq_state);
         st.update_well_var("P1", "WBHP", 1);
         st.update_well_var("P2", "WBHP", 2);
 
@@ -1362,15 +1400,12 @@ BOOST_AUTO_TEST_CASE(UDQ_TYPE_ERROR) {
             BOOST_CHECK_EQUAL(res2[index].get(), 3);
     }
 
-    parseContext.update(ParseContext::UDQ_TYPE_ERROR, InputError::THROW_EXCEPTION);
+    parseContext.update(ParseContext::UDQ_TYPE_ERROR, InputErrorAction::THROW_EXCEPTION);
 
     // This fails because the well expression (WBHP + 1) is assigned to the field variable FUBHP
     BOOST_CHECK_THROW( UDQDefine(udqp, "FUBHP",0, location, tokens1, parseContext, errors), OpmInputError);
 }
 
-
-
-
 BOOST_AUTO_TEST_CASE(UDA_VALUE) {
     UDAValue value0;
     BOOST_CHECK(value0.is<double>());
@@ -1403,10 +1438,7 @@ BOOST_AUTO_TEST_CASE(UDA_VALUE) {
     BOOST_CHECK_THROW( value2 *= 10, std::exception );
 }
 
-
-/*
-  The unit/dimension handling in the UDAvalue is hacky at best.
-*/
+// The unit/dimension handling in the UDAvalue is hacky at best.
 
 BOOST_AUTO_TEST_CASE(UDA_VALUE_DIM) {
     UDAValue value0(1);
@@ -1419,7 +1451,6 @@ BOOST_AUTO_TEST_CASE(UDA_VALUE_DIM) {
     BOOST_CHECK_EQUAL( value1.getSI(), 10);
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQ_INPUT_BASIC) {
     std::string deck_string = R"(
 SCHEDULE
@@ -1472,7 +1503,6 @@ UDQ
     BOOST_CHECK( wubhp1.is<UDQAssign>() );
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQ_INPUT_OVERWRITE) {
     std::string deck_string = R"(
 SCHEDULE
@@ -1510,7 +1540,6 @@ UDQ
     BOOST_CHECK_EQUAL(def2.input_string(), "MAX(WOPR)");
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQ_USAGE) {
     UDQActive usage;
     UDQParams params;
@@ -1661,7 +1690,8 @@ BOOST_AUTO_TEST_CASE(IntegrationTest) {
 }
 
 namespace {
-    Schedule make_udq_schedule(const std::string& schedule_string) {
+    Schedule make_udq_schedule(const std::string& schedule_string)
+    {
 #include "data/integration_tests/udq2.data"
         deck_string += schedule_string;
         return make_schedule(deck_string);
@@ -1798,7 +1828,6 @@ WCONPROD
     }
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQ_SCIENTIFIC_LITERAL) {
     std::string deck_string = R"(
 SCHEDULE
@@ -1820,7 +1849,6 @@ UDQ
     BOOST_CHECK_CLOSE( res0[0].get(), -0.00125*3, 1e-6);
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQ_NEGATIVE_PREFIX_BASIC) {
     std::string deck_string = R"(
 SCHEDULE
@@ -1882,7 +1910,8 @@ UDQ
     SummaryState st(TimeService::now());
     UDQFunctionTable udqft(udqp);
     UDQState udq_state(udqp.undefinedValue());
-    UDQContext context(udqft, WellMatcher(NameOrder({"W1", "W2", "W3"})), st, udq_state);
+    WellMatcher wm(NameOrder({"W1", "W2", "W3"}));
+    UDQContext context(udqft, wm, st, udq_state);
     st.update_well_var("W1", "WOPR", 1);
     st.update_well_var("W2", "WOPR", 2);
     st.update_well_var("W3", "WOPR", 3);
@@ -1902,7 +1931,6 @@ UDQ
     BOOST_CHECK_EQUAL( res1["W3"].get(), 0);
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQ_UADD_PARSER) {
     std::string deck_string = R"(
 SCHEDULE
@@ -1931,7 +1959,6 @@ UDQ
     BOOST_CHECK_EQUAL( st.get("FU_UMIN"), 5);    // min(10, 2+3)
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQ_DEFINE_ORDER) {
     std::string deck_string = R"(
 SCHEDULE
@@ -1972,9 +1999,6 @@ DEFINE FU_PAR3 FU_PAR2 + 1/
     BOOST_CHECK_EQUAL(st.get("FU_PAR3"), undefined_value);
 }
 
-
-
-
 BOOST_AUTO_TEST_CASE(UDQSTATE) {
     double undefined_value = 1234;
     UDQState st(undefined_value);
@@ -2013,8 +2037,6 @@ BOOST_AUTO_TEST_CASE(UDQSTATE) {
     BOOST_CHECK_EQUAL(st.get_well_var("P2", "WUPR"), undefined_value);
 }
 
-
-
 BOOST_AUTO_TEST_CASE(UDQ_UADD_PARSER2) {
     std::string deck_string = R"(
 SCHEDULE
@@ -2080,7 +2102,8 @@ DEFINE WUGASRA  750000 - WGLIR '*' /
     st.update_well_var("W3", "WGLIR", 3);
 
     NameOrder wo({"W1", "W2", "W3"});
-    udq.eval(0, WellMatcher(wo), st, udq_state);
+    WellMatcher wm(wo);
+    udq.eval(0, wm, st, udq_state);
     {
         std::unordered_set<std::string> required_keys;
         udq.required_summary(required_keys);
@@ -2095,7 +2118,6 @@ DEFINE WUGASRA  750000 - WGLIR '*' /
     }
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQ_UNDEFINED) {
     std::string deck_string = R"(
 SCHEDULE
@@ -2288,8 +2310,6 @@ DEFINE FU_VAR91 GOPR TEST  /
     udq.eval(0, {}, st, udq_state);
 }
 
-
-
 BOOST_AUTO_TEST_CASE(UDQ_KEY_ERROR) {
     std::string deck_string = R"(
 -- udq #2
@@ -2309,7 +2329,6 @@ UDQ
     BOOST_CHECK_THROW(udq.eval(0, {}, st, udq_state), std::exception);
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQ_ASSIGN) {
     std::string deck_string = R"(
 -- udq #2
@@ -2406,7 +2425,6 @@ TSTEP
     BOOST_CHECK( unique[2].second == schedule.getUDQConfig(10));
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQ_DIV_TEST) {
     KeywordLocation location;
     UDQFunctionTable udqft;
@@ -2420,7 +2438,6 @@ BOOST_AUTO_TEST_CASE(UDQ_DIV_TEST) {
     BOOST_CHECK_EQUAL( res_div[0].get() , 2.0);
 }
 
-
 BOOST_AUTO_TEST_CASE(UDQ_LEADING_SIGN) {
     std::string deck_string = R"(
 SCHEDULE
@@ -2640,7 +2657,8 @@ UDQ
     UDQState udq_state(0);
     SummaryState st(TimeService::now());
     UDQFunctionTable udqft;
-    UDQContext context(udqft, WellMatcher(NameOrder({"W1", "W2", "W3"})), st, udq_state);
+    WellMatcher wm(NameOrder({"W1", "W2", "W3"}));
+    UDQContext context(udqft, wm, st, udq_state);
     st.update_well_var("W1", "WBHP", 400);
     st.update_well_var("W2", "WBHP", 300);
     st.update_well_var("W3", "WBHP", 200);
@@ -2673,8 +2691,6 @@ UDQ
     BOOST_CHECK_NO_THROW( make_schedule(valid) );
 }
 
-
-
 BOOST_AUTO_TEST_CASE(UDQ_ASSIGN_RST) {
     std::unordered_set<std::string> selector{"W1", "W2"};
     UDQAssign assign("WUBHP", selector, 100, 2);
